#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "config.h"
#include "notes.h"
#include "linux_audio.h"
#include "pitchdetector.h"

void print_spectrogram(double freq, double amp);
void write_sine(int dsp_fd, double freq, unsigned num_samples);

int main()
{
	PitchDetector pd(SAMPLE_RATE, FFT_LENGTH, PAD_FACTOR, OVERLAP);
	int fd = get_dsp_fd(SAMPLE_RATE, FFT_LENGTH, OVERLAP);
	for ( ;; ) {
		short buf[FFT_LENGTH / PAD_FACTOR / OVERLAP];

		read_chunk(fd, buf, FFT_LENGTH / PAD_FACTOR / OVERLAP);
		std::pair<double, double> peak = pd.detect_pitch(buf);

		if (peak.first < 50.0 || peak.second - log10(FFT_LENGTH) < 0.0) {
#if TUNING == WELL_TEMPERED_GUITAR
			printf("......\n");
#else		
			printf("............\n");
#endif
		} else {
			print_spectrogram(peak.first, peak.second - log10(FFT_LENGTH));
		}
	}
}

std::string freq_to_tonename(double freq)
{
	std::string notenames[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
	double half_notes_away = 12.0 * log2(freq / BASE_PITCH) - 3.0;
	int hnai = int(floor(half_notes_away + 0.5));
	int octave = (hnai + 48) / 12;

	char buf[256];
	sprintf(buf, "%s%d + %.2f [%d]", notenames[((hnai % 12) + 12) % 12].c_str(), octave, half_notes_away - hnai, hnai);
	return buf;
}

#if TUNING == EQUAL_TEMPERAMENT
void print_spectrogram(double freq, double amp)
{
	std::string notenames[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
	double half_notes_away = 12.0 * log2(freq / BASE_PITCH) - 3.0;
	int hnai = int(floor(half_notes_away + 0.5));
	int octave = (hnai + 48) / 12;

	for (int i = 0; i < 12; ++i)
		if (i == ((hnai % 12) + 12) % 12)
			printf("#");
		else
			printf(".");

	printf(" (%-2s%d %+.2f, %5.2fHz) [%5.2fdB]  [", notenames[((hnai % 12) + 12) % 12].c_str(), octave, half_notes_away - hnai,
		freq, amp);

	double off = half_notes_away - hnai;
	for (int i = -10; i <= 10; ++i) {
		if (off >= (i-0.5) * 0.05 && off < (i+0.5) * 0.05) {
			printf("#");
		} else {
			if (i == 0) {
				printf("|");
			} else {
				printf("-");
			}
		}
	}
	printf("]\n");

}
#else
void print_spectrogram(double freq, double amp)
{
	double best_away = 999999999.9;
	unsigned best_away_ind = 0;
	
	for (unsigned i = 0; i < sizeof(notes)/sizeof(note); ++i) {
		double half_notes_away = 12.0 * log2(freq / notes[i].freq);
		if (fabs(half_notes_away) < fabs(best_away)) {
			best_away = half_notes_away;
			best_away_ind = i;
		}
	}
	
	for (unsigned i = 0; i < sizeof(notes)/sizeof(note); ++i)
		if (i == best_away_ind)
			printf("#");
		else
			printf(".");

	printf(" (%s %+.2f, %5.2fHz) [%5.2fdB]  [", notes[best_away_ind].notename, best_away, freq, amp);

	// coarse tuning
	for (int i = -10; i <= 10; ++i) {
		if (best_away >= (i-0.5) * 0.05 && best_away < (i+0.5) * 0.05) {
			printf("#");
		} else {
			if (i == 0) {
				printf("|");
			} else {
				printf("-");
			}
		}
	}
	printf("]  [");
	
	// fine tuning
	for (int i = -10; i <= 10; ++i) {
		if (best_away >= (i-0.5) * 0.01 && best_away < (i+0.5) * 0.01) {
			printf("#");
		} else {
			if (i == 0) {
				printf("|");
			} else {
				printf("-");
			}
		}
	}
	printf("]\n");
}
#endif
